/// <reference types="./select-parser.d.ts" />
import { isReadonlyArray, isString } from '../util/object-utils.js';
import { SelectionNode } from '../operation-node/selection-node.js';
import { parseAliasedStringReference } from './reference-parser.js';
import { isDynamicReferenceBuilder, } from '../dynamic/dynamic-reference-builder.js';
import { parseAliasedComplexExpression, } from './complex-expression-parser.js';
export function parseSelectExpressionOrList(ctx, selection) {
    if (isReadonlyArray(selection)) {
        return selection.map((it) => parseSelectExpression(ctx, it));
    }
    else {
        return [parseSelectExpression(ctx, selection)];
    }
}
function parseSelectExpression(ctx, selection) {
    if (isString(selection)) {
        return SelectionNode.create(parseAliasedStringReference(selection));
    }
    else if (isDynamicReferenceBuilder(selection)) {
        return SelectionNode.create(selection.toOperationNode());
    }
    else {
        return SelectionNode.create(parseAliasedComplexExpression(ctx, selection));
    }
}
export function parseSelectAll(table) {
    if (!table) {
        return [SelectionNode.createSelectAll()];
    }
    else if (Array.isArray(table)) {
        return table.map(parseSelectAllArg);
    }
    else {
        return [parseSelectAllArg(table)];
    }
}
function parseSelectAllArg(table) {
    if (isString(table)) {
        return SelectionNode.createSelectAllFromTable(table);
    }
    throw new Error(`invalid value selectAll expression: ${JSON.stringify(table)}`);
}
